﻿using Syncfusion.UI.Xaml.Controls.Input;
using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.Grid.Cells;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace MasterDetailsViewSingleLevel
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();            
            this.dataGrid.DetailsViewLoading += DataGrid_DetailsViewLoading;
            this.dataGrid.ResizingColumns += DataGrid_ResizingColumns;
        }

        private void DataGrid_ResizingColumns(object sender, ResizingColumnsEventArgs e)
        {
            var grid = sender as SfDataGrid;
            if (e.OriginalSender is DetailsViewDataGrid)
                grid = e.OriginalSender as SfDataGrid;
            if (grid.View == null)
                return;
            SetWidth(grid, e.ColumnIndex, e.Width);
        }

        private void SetWidth(SfDataGrid grid, int scrollColumnIndex, double width)
        {
            if (grid.DetailsViewDefinition == null || !grid.DetailsViewDefinition.Any())
                return;
            if (!CanResize(grid))
                return;
            var columnIndex = grid.HelperResolveToGridVisibleColumnIndex(scrollColumnIndex);
            if (columnIndex < 0)
                return;
            var parentstartcolumnnIndex = grid.HelperResolveToStartColumnIndex();
            var indentcolumnsWidth = 0;
            foreach (var definition in grid.DetailsViewDefinition)
            {
                var detailsViewDataGrid = (definition as GridViewDefinition).DataGrid;
                var startcolumnnIndex = detailsViewDataGrid.HelperResolveToStartColumnIndex();
                indentcolumnsWidth = startcolumnnIndex * 24;
                var tempWidth = width - indentcolumnsWidth < 0 ? 0 : width - indentcolumnsWidth;
                detailsViewDataGrid.Columns[columnIndex].Width = scrollColumnIndex == parentstartcolumnnIndex ? tempWidth : width;
                // If DetailsViewDataGrid has DetailsViewDefinition(nested levels), recursively set width upto all levels
                if (detailsViewDataGrid.DetailsViewDefinition != null && detailsViewDataGrid.DetailsViewDefinition.Any())
                    SetWidth(detailsViewDataGrid, detailsViewDataGrid.HelperResolveToScrollColumnIndex(columnIndex), detailsViewDataGrid.Columns[columnIndex].Width);
            }

        }

        private bool CanResize(SfDataGrid dataGrid)
        {
            if (dataGrid.DetailsViewDefinition == null && !dataGrid.DetailsViewDefinition.Any())
                return true;
            foreach (var definition in dataGrid.DetailsViewDefinition)
            {
                var detailsViewGrid = (definition as GridViewDefinition).DataGrid;
                if (detailsViewGrid.DetailsViewDefinition == null && !detailsViewGrid.DetailsViewDefinition.Any())
                    return CanResize(detailsViewGrid);
                if (detailsViewGrid.Columns.Count != dataGrid.Columns.Count)
                    return false;
            }
            return true;
        }

        private void DataGrid_DetailsViewLoading(object sender, Syncfusion.UI.Xaml.Grid.DetailsViewLoadingAndUnloadingEventArgs e)
        {
            var parentGrid = e.OriginalSender is DetailsViewDataGrid ? (e.OriginalSender as SfDataGrid) : sender as SfDataGrid;
            if (!CanResize(parentGrid))
                return;
            if (parentGrid.Columns.Count != e.DetailsViewDataGrid.Columns.Count)
                return;
            double width = 0;
            var detailsViewStartColumnIndex = e.DetailsViewDataGrid.ResolveToStartColumnIndex();
            for (int i = 0; i < parentGrid.Columns.Count; i++)
            {
                width = i == 0 ? parentGrid.Columns[i].ActualWidth - detailsViewStartColumnIndex * 24 : parentGrid.Columns[i].Width;
                if (e.DetailsViewDataGrid.Columns[i].Width != parentGrid.Columns[i].Width)
                    e.DetailsViewDataGrid.Columns[i].Width = width;
            }

        }     
    }

    public static class GridHelperClass
    {
        /// <summary>
        /// Resolves the start column index of the ViewDefinition.DataGrid.
        /// </summary>
        /// <param name="dataGrid">
        /// The ViewDefinition.DataGrid
        /// </param>
        /// <returns>
        /// Returns the start column index of the ViewDefinition.DataGrid.
        /// </returns>
        public static int HelperResolveToStartColumnIndex(this SfDataGrid dataGrid)
        {
            int startIndex = 0;
            if (dataGrid.ShowRowHeader)
                startIndex += 1;
            if (dataGrid.GroupColumnDescriptions != null && dataGrid.GroupColumnDescriptions.Any())
                startIndex += dataGrid.GroupColumnDescriptions.Count;
            if (dataGrid.DetailsViewDefinition != null && dataGrid.DetailsViewDefinition.Any())
                startIndex += 1;
            return startIndex;
        }

        /// <summary>
        /// Resolves the visible column index for the specified column index in ViewDefinition.DataGrid.
        /// </summary>
        /// <param name="dataGrid">
        /// The ViewDefinition.DataGrid.
        /// </param>
        /// <param name="visibleColumnIndex">
        /// The visibleColumnIndex.
        /// </param>
        /// <returns>
        /// Returns the corresponding visible column index for the specified column index.
        /// </returns>
        public static int HelperResolveToGridVisibleColumnIndex(this SfDataGrid dataGrid, int visibleColumnIndex)
        {
            var indentColumnCount = (dataGrid.GroupColumnDescriptions != null ? dataGrid.GroupColumnDescriptions.Count : 0) +
                ((dataGrid.DetailsViewDefinition != null && dataGrid.DetailsViewDefinition.Any()) ? 1 : 0);
            int resolvedIndex = visibleColumnIndex - (indentColumnCount + (dataGrid.ShowRowHeader ? 1 : 0));
            return resolvedIndex;
        }

        /// <summary>
        /// Resolves the scroll column index for the specified column index in ViewDefinition.DataGrid.
        /// </summary>
        /// <param name="dataGrid">
        /// The ViewDefinition.DataGrid.
        /// </param>
        /// <param name="gridColumnIndex">
        /// The corresponding column index to get the scroll column index.
        /// </param>
        /// <returns>
        /// Returns the scroll column index for the specified column index.
        /// </returns>
        public static int HelperResolveToScrollColumnIndex(this SfDataGrid dataGrid, int gridColumnIndex)
        {
            var indentColumnCount = ((dataGrid.DetailsViewDefinition != null && dataGrid.DetailsViewDefinition.Any()) ? 1 : 0) +
                (dataGrid.GroupColumnDescriptions != null ? dataGrid.GroupColumnDescriptions.Count : 0);
            return ((dataGrid.ShowRowHeader ? 1 : 0) + indentColumnCount) + gridColumnIndex;
        }
    }


    public class OrderInfo
    {
        int orderID;
        string customerId;
        string country;
        List<ProductInfo> productDetails;

        public int OrderID
        {
            get { return orderID; }
            set { orderID = value; }
        }

        public string CustomerID
        {
            get { return customerId; }
            set { customerId = value; }
        }


        public string Country
        {
            get { return country; }
            set { country = value; }
        }

        public List<ProductInfo> ProductDetails
        {
            get { return productDetails; }
            set { productDetails = value; }
        }


        public OrderInfo(int orderId, string customerName, string country, string customerId, string shipCity, List<ProductInfo> productdetails)
        {
            this.OrderID = orderId;            
            this.Country = country;
            this.CustomerID = customerId;            
            this.ProductDetails = productdetails;
        }
    }
    public class ProductInfo
    {
        int orderId;
        string productName;
        DateTime dateOfMonth;

        public int OrderID
        {
            get { return orderId; }
            set { orderId = value; }
        }

        [Required()]
        public string ProductName
        {
            get { return productName; }
            set { productName = value; }
        }

        public DateTime DateOfMonth
        {
            get { return dateOfMonth; }
            set { dateOfMonth = value; }
        }
    }
    public class OrderInfoRepositiory
    {
        ObservableCollection<OrderInfo> orders;
        public ObservableCollection<OrderInfo> Orders
        {
            get { return orders; }
            set { orders = value; }
        }

        public OrderInfoRepositiory()
        {
            orders = new ObservableCollection<OrderInfo>();
            this.GenerateProducts();
            Orders = GenerateOrders();
        }

        public ObservableCollection<OrderInfo> GenerateOrders()
        {
            orders.Add(new OrderInfo(1001, "Maria Anders", "Germany", "ALFKI", "Berlin", getorder(1001)));
            orders.Add(new OrderInfo(1002, "Ana Trujilo", "Mexico", "ANATR", "México D.F.", getorder(1002)));
            orders.Add(new OrderInfo(1003, "Antonio Moreno", "Mexico", "ANTON", "México D.F.", getorder(1003)));
            orders.Add(new OrderInfo(1004, "Thomas Hardy", "UK", "AROUT", "London", getorder(1004)));
            orders.Add(new OrderInfo(1005, "Christina Berglund", "Sweden", "BERGS", "Luleå", getorder(1005)));
            orders.Add(new OrderInfo(1006, "Hanna Moos", "Germany", "BLAUS", "Mannheim", getorder(1006)));
            orders.Add(new OrderInfo(1007, "Frédérique Citeaux", "France", "BLONP", "Strasbourg", getorder(1007)));
            orders.Add(new OrderInfo(1008, "Martin Sommer", "Spain", "BOLID", "Madrid", getorder(1008)));
            orders.Add(new OrderInfo(1009, "Laurence Lebihan", "France", "BONAP", "Marseille", getorder(1009)));
            orders.Add(new OrderInfo(1010, "Elizabeth Lincoln", "Canada", "BOTTM", "Tsawassen", getorder(1010)));
            return orders;
        }

        List<ProductInfo> products = new List<ProductInfo>();

        public void GenerateProducts()
        {
            products.Add(new ProductInfo() { OrderID = 1001, ProductName = "", DateOfMonth = new DateTime(2015, 5, 27) });
            products.Add(new ProductInfo() { OrderID = 1001, ProductName = "Bike2", DateOfMonth = new DateTime(2015, 5, 20) });
            products.Add(new ProductInfo() { OrderID = 1001, ProductName = "Bike2", DateOfMonth = new DateTime(2015, 5, 27) });
            products.Add(new ProductInfo() { OrderID = 1002, ProductName = "Bike5", DateOfMonth = new DateTime(2014, 5, 7) });
            products.Add(new ProductInfo() { OrderID = 1002, ProductName = "", DateOfMonth = new DateTime(2012, 5, 20) });
            products.Add(new ProductInfo() { OrderID = 1002, ProductName = "Bike3", DateOfMonth = new DateTime(2015, 5, 27) });
            products.Add(new ProductInfo() { OrderID = 1003, ProductName = "Bike2", DateOfMonth = new DateTime(2015, 5, 27) });
            products.Add(new ProductInfo() { OrderID = 1003, ProductName = "Bike1" });
            products.Add(new ProductInfo() { OrderID = 1004, ProductName = "Bike2" });
            products.Add(new ProductInfo() { OrderID = 1004, ProductName = "Bike2" });
            products.Add(new ProductInfo() { OrderID = 1005, ProductName = "Bike3" });
            products.Add(new ProductInfo() { OrderID = 1005, ProductName = "Bike4" });
            products.Add(new ProductInfo() { OrderID = 1005, ProductName = "Bike5" });
            products.Add(new ProductInfo() { OrderID = 1006, ProductName = "Bike1" });
            products.Add(new ProductInfo() { OrderID = 1006, ProductName = "Bike2" });
            products.Add(new ProductInfo() { OrderID = 1007, ProductName = "Bike1" });
            products.Add(new ProductInfo() { OrderID = 1008, ProductName = "Bike4" });
            products.Add(new ProductInfo() { OrderID = 1008, ProductName = "Bike6" });
            products.Add(new ProductInfo() { OrderID = 1008, ProductName = "Bike7" });
        }

        public List<ProductInfo> getorder(int i)
        {
            List<ProductInfo> product = new List<ProductInfo>();
            foreach (var or in products)
                if (or.OrderID == i)
                    product.Add(or);
            return product;
        }
    }  
}
